@@ -10,6 +10,7 @@ import mx.Tidy
 
 from config import java
 from util import newTempfile, runcmd
+from elementtree.ElementTree import parse, tostring        
 
 from zope.interface import implements
 from interfaces import IHTML2FOConverter
@@ -24,7 +25,7 @@ def tidyhtml(filename, encoding):
 
     html = open(filename, 'rb').read()
     nerrors, nwarnings, html, errordata = mx.Tidy.tidy(html,
-                                                       drop_empty_paras=0,
+                                                       drop_empty_paras=1,
                                                        char_encoding=encoding,
                                                        add_xml_decl=1,
                                                        output_xhtml=1)
@@ -62,31 +63,40 @@ class HTML2FOConverter(object):
 
         # remove some stuff from the generated FO file causing
         # some conversion trouble either with XINC or XFC
-        fo_text = open(fo_filename, 'rb').read()
 
-        reg_s = '(footnote="reset")|'\
-                '(unicode-bidi="embed")|'\
-                '(writing-mode="lr-tb")|'\
-                '(page-break-after="avoid")|'\
-                '(page-break-before="avoid")|'\
-                '(page-break-inside="avoid")'\
+        E = parse(fo_filename)
 
-        reg = re.compile(reg_s, re.I|re.S|re.M)
-        fo_text = reg.sub('', fo_text)
+        for node in E.getiterator():
+            
+            for k, v in (('footnote', 'reset'), 
+                         ('unicode-bidi', 'embed'), 
+                         ('writing-mode', 'lr-tb'), 
+                         ('font-selection-strategy', 'character-by-character'), 
+                         ('line-height-shift-adjustment', 'disregard-shifts'), 
+                         ('page-break-after', 'avoid'), 
+                         ('page-break-before', 'avoid'), 
+                         ('page-break-inside', 'avoid')):
 
-        fo_text = fo_text.replace('<fo:block ' , '<fo:block margin-top="0" margin-bottom="0" ')  # avoid a linebreak through <li><p> (XFC)
-        fo_text = fo_text.replace('<fo:block/>', '') # causes a crash with XINC    
-        fo_text = fo_text.replace('page-break-after="always"', 'break-after="page"')
-        
+                value = node.attrib.get(k)
+                if value == v:
+                    del node.attrib[k]
 
-        # replace margin|padding-left|right|top-bottom="0" with ..="0em"
-        def replacer(mo):
-            return '%s-%s="0em"' % (mo.group(1), mo.group(2))
+            for i in ('margin', 'padding'):
+                for j in ('left', 'right', 'top', 'bottom'):
+                    attr = i + '-'  + j
+                    
+                    value = node.attrib.get(attr)
+                    if value == '0':
+                        node.attribute = '0em'
 
-        reg = re.compile('(margin|padding)-(left|right|top|bottom)="0"', re.I|re.S|re.M)
-        fo_text = reg.sub(replacer, fo_text)
 
-        print fo_filename
+            if node.attrib.get('page-break-after') == 'always':
+                del node.attrib['page-break-after']
+                node.attrib['break-after'] = 'page'
+                
+        fo_text = tostring(E.getroot())
+        fo_text = fo_text.replace('<ns0:block ' , '<ns0:block margin-top="0" margin-bottom="0" ')  # avoid a linebreak through <li><p> (XFC)
+        fo_text = fo_text.replace('<ns0:block/>', '') # causes a crash with XINC    
 
         open(fo_filename, 'wb').write(fo_text)
         return fo_filename
